#include "dm.h"
#define TOKEN_LENGTH 100


DM::DM(char *mapfile, char *datafile, char *targetfile){

  line_size=0;

  read_full_map(mapfile);
  read_data(datafile);
  read_target_list(targetfile);
  if(ilist.size()==0){
    int cur=0;
    for(int i=0;i<tlist.size();i++){
      for(int j=cur;j<tlist[i];j++)
	ilist.push_back(j);   
      cur = tlist[i]+1;
    }
    for(int i=cur;i<full_map.size();i++)
      ilist.push_back(i); 
  }
 
}
char* DM::get_token(FILE *fp, int & status ){

  char* token = new char[TOKEN_LENGTH];
  memset(token,0,TOKEN_LENGTH);
  char  in;
  int count=0;
  int start=0;

  status = 0;
  
  while(1){
    // fetch next character in the stream
    in = fgetc(fp);
    
    if(isalnum(in)||ispunct(in)){
      if(!start)
	start=1;
      token[count++]=in;
    }
    if(in=='\n'||in=='\r'){
      status=1;
      return token;
    }
    
    // meet the delimiter
    if(isspace(in)&&start){
      return token;
    }
    
    if(in == EOF){
      status = 2;
      return token;
    }
    
  }
}

void DM::read_full_map(char *filename){
  
  ifstream tlist(filename);
  istringstream ins;
  string line;
 
  string marker;
  int pos;
  
  char alleleA;
  char alleleB;
  
  int context_start = 0;
  int order = 0;
  
  while (getline(tlist,line) ){    
    
    ins.clear();
    ins.str(line);
    
    ins>>ws;
    
    // empty line, skip
    if(ins.eof())
      continue;
    
    if(ins >> marker >> pos >>alleleA >> alleleB){
      
      if(context_start==0)
	context_start = 1;

      //physical distance
      physical_map[order] = pos;           
      //map number to marker name
      reverse_map[order] = marker;
      // map marker name to order
      full_map[marker] = order++;
      
    }else{
      
      if(context_start==0){
	// possible header, skip it
	context_start=1;
	continue;
      }
      
      fprintf(stderr, "Error: Unexpected format in full map file\n");
      exit(1);
    }
    
  }
  
  tlist.close();
}



void DM::read_target_list(char *filename){

  if(filename==0)
    return;
  
  ifstream tlist(filename);
  istringstream ins;
  string line;
  
  string marker;
  while (getline(tlist,line) ){    
    
    ins.clear();
    ins.str(line);
    
    ins>>ws;
    
    // empty line, skip
    if(ins.eof())
      continue;
    if(ins >> marker){
      if(full_map.find(marker)==full_map.end())
	fprintf(stderr, "Skip SNP %s in target list: marker is not defined in full map\n",marker.c_str()); 
      else
	ilist.push_back(full_map[marker]);
    }
    
  }
  tlist.close();

}


void DM::read_data(char *data_file){
  
  FILE *fp = fopen(data_file,"r");
  if(fp==0)
    return;
  
  vector<string> line;
  int status;
  char *token;
  int marker_flag = 0;
  int ignore = 0;
  vector<double> mv;
  
  
  while(1){
    
    token = get_token(fp,status);
    
    if(strlen(token) != 0){
      // first token in the line
      if(!marker_flag){
	string name(token);
	if(full_map.find(name) == full_map.end()){
	  ignore = 1;
	  fprintf(stderr,"Ignore SNP %s in input data: marker is not defined in full map\n",token);
	}else
	  tlist.push_back(full_map[name]);
	marker_flag = 1;
      }else{
	if(!ignore)
	  mv.push_back(atof(token));
      }
      
    }

    delete[] token;
    
    if(status==1){
      
      // check sample size
      if(line_size==0)
	line_size = mv.size();
      else if(line_size != mv.size()){
	fprintf(stderr, "Error: Unequal number of SNPs recorded in input data file %d %d\n",line_size, mv.size());
	return;
      }
      ignore = 0;
      marker_flag = 0;
      double *cl = new double[mv.size()];
      for(int i=0;i<mv.size();i++)
	cl[i] = mv[i];
      mv.clear();
      obsv.push_back(cl);
      
    }
    
    if(status==2)
      break;
    
  }

  return;

  
}

