#include "lpac.h"


void print_matrix(gsl_matrix *matrix){
   for(int i=0;i<matrix->size1;i++){
     for(int j=0;j<matrix->size2;j++)
       printf("%7.4e ",gsl_matrix_get(matrix,i,j));
     printf("\n");
   }
   printf("\n\n");
}




void find_bounds(gsl_matrix *tcv,vector<int>& lf, vector<int> &rt){
  int n = tcv->size1;
  int m = tcv->size2;
  
  for(int i=0;i<n;i++){
    int val = m-1;
    for(int j=0;j<m;j++){
      if(gsl_matrix_get(tcv,i,j)!=0){
	val = j;
	break;
      }
    }
    lf.push_back(val);
  }

	
  for(int i=0;i<n;i++){
    int val = 0;
    for(int j=m-1;j>=0;j--){
      if(gsl_matrix_get(tcv,i,j)!=0){
	val = j;
	break;
      }
    }
    rt.push_back(val);
  }

}



//Inversion using Gaussian elimination for banded matrix
gsl_matrix * ge_invert(gsl_matrix *target, vector<int> &lb, vector<int> &rb){
  
  int size = target->size1;
  gsl_matrix *work = gsl_matrix_calloc(size,2*size);
  for(int i=0;i<size;i++){
    gsl_matrix_set(work,i,size+i,1);
    for(int j=0;j<size;j++)
      gsl_matrix_set(work,i,j,gsl_matrix_get(target,i,j));
  }

  //forward elimination
  fprintf(stderr,"Forward elimination...\n");
  for (int i=0;i<size;i++)
    for (int j=i+1;j<=rb[i];j++){
      // find pivot
      double pivot = gsl_matrix_get(work,j,i)/gsl_matrix_get(work,i,i);
      for(int k=i;k<=rb[i];k++)
        gsl_matrix_set(work,j,k,gsl_matrix_get(work,j,k)-gsl_matrix_get(work,i,k)*pivot);
      //for(int k=0;k<size;k++)
      for(int k=0;k<=rb[i];k++)
        gsl_matrix_set(work,j,k+size,gsl_matrix_get(work,j,k+size)-gsl_matrix_get(work,i,k+size)*pivot);

    }

  fprintf(stderr,"Backward substitution...\n");
  // backward substitution
  for(int i=size-1;i>=0;i--)
    for(int j=i-1;j>=lb[i];j--){
      double pivot = gsl_matrix_get(work,j,i)/gsl_matrix_get(work,i,i);
      for(int k=j;k<=rb[i];k++)
        gsl_matrix_set(work,j,k,gsl_matrix_get(work,j,k)-gsl_matrix_get(work,i,k)*pivot);
      for(int k=size;k<2*size;k++)
        gsl_matrix_set(work,j,k,gsl_matrix_get(work,j,k)-gsl_matrix_get(work,i,k)*pivot);
    }

  fprintf(stderr,"Inversion completed.\n\n");
  
  gsl_matrix *rst =  gsl_matrix_calloc(size,size); 
  for(int i=0;i<size;i++)
    for(int j=0;j<size;j++){
      double val = gsl_matrix_get(work,i,i);
      gsl_matrix_set(rst,i,j,gsl_matrix_get(work,i,size+j)/val);
    }
    
  gsl_matrix_free(work);
  return rst;
}



// compute matrix product AB while A is sparse, and band info for A is given by (lb,rb)
gsl_matrix *sparse_matrix_mult1(gsl_matrix *A, gsl_matrix *B, vector<int> & lb, vector<int> &rb){
 
  int row = A->size1;
  int loop = A->size2;
  int col = B->size2;

  gsl_matrix *rst = gsl_matrix_calloc(row,col);
  for( int i=0;i<row;i++){
    for(int j=0;j<col;j++){
      double sum = 0;
      for (int k=lb[i];k<=rb[i];k++)
	sum += gsl_matrix_get(A,i,k)*gsl_matrix_get(B,k,j);
      gsl_matrix_set(rst,i,j,sum);
    }
  }
  return rst;
}

// compute matrix product diag(A t(B)) while A and B are sparse, and band info for A,B is given by (lba,rba) and (lbb,rbb)
gsl_matrix *sparse_matrix_mult2_diag(gsl_matrix *A,gsl_matrix *B, vector<int> &lba,vector<int> &rba,vector<int> &lbb,vector<int> &rbb){
  int n = B->size1;
  int m = B->size2;
  gsl_matrix *rst = gsl_matrix_calloc(n,1);
  
  for(int i=0;i<n;i++){   
    int lb = (lba[i]>lbb[i])? lba[i]:lbb[i];
    int rb = (rba[i]<rbb[i])? rba[i]:rbb[i];
    double sum = 0;
    for(int k=lb;k<=rb;k++)
      sum += gsl_matrix_get(A,i,k)*gsl_matrix_get(B,i,k);
    gsl_matrix_set(rst,i,0,sum);
  }
  return rst;
}


