#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <gsl/gsl_cdf.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>

#include <vector>
#include <map>
#include <algorithm>
#include <string>

#define TOKEN_LENGTH 100

using namespace::std;

class LPAC{

 public:
  vector<int> ilist;
  
 private:
  gsl_matrix *cvar_oo;
  gsl_matrix *mean_u;
  gsl_matrix *mean_o;
  gsl_matrix *cvar_uo;
  gsl_matrix *diff;
  
  vector<double *> obsv;
  vector<int> tlist;
  map<int,int> tmap;
  
  map<int,string> rsmap;
  
  double mutp;
  double thresh;    
  
  char **matrix;
  int panel_size;
  int sample_size;
  int marker_num;
  int lsize;
  vector<double> rmb;

  //settings
  int panel_type; // 0 for phased panel, 1 for unphased panel
  int pred_type;  // 0 for freq imputation, 1 for geno imputation
  
  //parameters
  double sigma;
  double delta;


  FILE *outf;

  public:
    
  LPAC(vector<int>& typed_list, vector<int>& impute_list,char *panel_data, char *rmb_data, vector<double *> &obs,int line_size, map<int,string>& rslist,int ptype);
    void setupCovar(int start, int end);
    void predict(int start, int end);

    void estimate_params();
    void estimate_sigma(double input_delta);
    void estimate_delta(double input_sigma);
    
    void set_params(double ns, double nd){ delta = nd; sigma = ns; };
    
    void set_imputation_type(int type) { pred_type = type; };


    int parse_panel(char *filename);
    int parse_recomb (char * rmb_file);       

    double compute_coef(int pos1, int pos2);
    double compute_freq(int pos);
    double compute_D(int pos1, int pos2);
    
    void set_outfile(char *outfile);

}; 




//utilities
void print_matrix(gsl_matrix *matrix);


//band matrix
void find_bounds(gsl_matrix *tcv,vector<int>& lf, vector<int> &rt);


gsl_matrix * ge_invert(gsl_matrix *target, vector<int> &lb, vector<int> &rb);
gsl_matrix *sparse_matrix_mult1(gsl_matrix *A, gsl_matrix *B, vector<int> &lb, vector<int> &rb);
gsl_matrix *sparse_matrix_mult2_diag(gsl_matrix *A,gsl_matrix *B, vector<int> &lba,vector<int> &rba,vector<int> &lbb,vector<int> &rbb);



