#include "lpac.h"
#include "dm.h"



void print_banner(){

  fprintf(stderr, "\n----------------------------------------------\n");
  fprintf(stderr, "      BLIMP: Best Linear IMPutation                  \n");
  fprintf(stderr, "          by X.Wen and M.Stephens                 \n");
  fprintf(stderr, "            Version 11.2 beta                     \n");
  fprintf(stderr, "----------------------------------------------\n\n\n");
}




int main(int argc, char **argv){

  
  print_banner();
  // parse command line
  
  // files
  char *panel = 0;
  char *rmb = 0;
  char *pmap = 0;
  char *outfile = 0;
  char *data=0;
  char *target=0;
  
  //settings
  
  int pred_type = 0;  // 0 for freq, 1 for geno
  int panel_type = 0; // 0 for phased panel, 1 for unphased panel
  
  double sigma = -1;
  double delta=  0;

  for(int i=1;i<argc;i++){
    if(strcmp(argv[i],"-p")==0){
      panel = argv[++i];
      continue;
    }
    if(strcmp(argv[i],"-r")==0){
      rmb = argv[++i];
      continue;
    }
    if(strcmp(argv[i],"-m")==0){
      pmap = argv[++i];
      continue;
    }
    
    if(strcmp(argv[i],"-d")==0){
      data = argv[++i];
      continue;
    }

    if(strcmp(argv[i],"-t")==0){
      target = argv[++i];
      continue;
    }
    
    if(strcmp(argv[i],"-geno")==0){
      pred_type = 1;
      continue;
    }
    
    // default
    if(strcmp(argv[i],"-freq")==0){
      pred_type = 0;
      continue;
    }
    
    if(strcmp(argv[i],"-phasep")==0){
      panel_type = 0;
      continue;
    }
    
    if(strcmp(argv[i],"-unphasep")==0){
      panel_type = 1;
      continue;
    }

    if(strcmp(argv[i],"-o")==0){
      outfile = argv[++i];
    }

    else{
      fprintf(stderr, "Error: unknown option \"%s\"\n",argv[i]);
      exit(1);
    }
    
    
  }
  

  DM dm(pmap,data,target);
  LPAC ld(dm.tlist, dm.ilist,panel,rmb,dm.obsv,dm.line_size,dm.reverse_map,panel_type);
  ld.set_imputation_type(pred_type);
  ld.set_outfile(outfile);


  vector<int> &ilist = dm.ilist;
  map<int,int>& map = dm.physical_map;
  
  
  ld.estimate_sigma(delta);
  
  int start = 0;
  int end;
  int gap = 5000000;
  while(1){

     if(start>=ilist.size())
      break;
     int sp = map[ilist[start]];
     int i = start;
     for(i=start;i<ilist.size();i++)
        if(map[ilist[i]]>gap+sp)
          break;        
    end = i;
    if(end==ilist.size())
        end--;
    ld.setupCovar(start,end);
    ld.predict(start,end);
    start = end+1;
  }

}

